# Generated by Django 5.0.3 on 2024-03-15 23:17

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('translator', '0017_remove_translated_content_id_and_more'),
    ]

    operations = [
        migrations.AlterField(
            model_name='claudetranslator',
            name='model',
            field=models.CharField(choices=[('claude-instant-1.2', 'claude-instant-1.2'), ('claude-2.1', 'claude-2.1'), ('claude-2.0', 'claude-2.0'), ('claude-3-opus-20240229', 'claude-3-opus-20240229'), ('claude-3-sonnet-20240229', 'claude-3-sonnet-20240229')], default='claude-instant-1.2', max_length=50),
        ),
    ]



import logging
from django.db import models
from django.utils.translation import gettext_lazy as _
import cityhash

class TranslatorEngine(models.Model):
    name = models.CharField(_("Name"), max_length=100, unique=True)
    valid = models.BooleanField(_("Valid"), null=True)

    def translate(self, text: str, target_language: str) -> dict:
        raise NotImplementedError(
            "subclasses of TranslatorEngine must provide a translate() method"
        )

    def min_size(self) -> int:
        if hasattr(self, "max_characters"):
            return self.max_characters * 0.7
        if hasattr(self, "max_tokens"):
            return self.max_tokens * 0.7
        return 0

    def max_size(self) -> int:
        if hasattr(self, "max_characters"):
            return self.max_characters * 0.9
        if hasattr(self, "max_tokens"):
            return self.max_tokens * 0.9
        return 0
    def validate(self) -> bool:
        raise NotImplementedError(
            "subclasses of TranslatorEngine must provide a validate() method"
        )

    class Meta:
        abstract = True

    def __str__(self):
        return self.name


class Translated_Content(models.Model):
    hash = models.BinaryField(max_length=8, unique=True, primary_key=True, editable=False)
    original_content = models.TextField()

    translated_language = models.CharField(max_length=255)
    translated_content = models.TextField()

    tokens = models.IntegerField(default=0)
    characters = models.IntegerField(default=0)

    def __str__(self):
        return self.original_content

    @classmethod
    def is_translated(cls, text, target_language):
        text_hash = cityhash.CityHash64(f"{text}{target_language}").to_bytes(8, byteorder='little')
        try:
            content = Translated_Content.objects.get(hash=text_hash)
            # logging.info("Using cached translations:%s", text)
            return {
                'text': content.translated_content,
                'tokens': content.tokens,
                'characters': content.characters
            }
        except Translated_Content.DoesNotExist:
            logging.info("Does not exist in cache:%s", text)
            return None

    def save(self, *args, **kwargs):
        if not self.hash:
            self.hash = cityhash.CityHash64(f"{self.original_content}{self.translated_language}").to_bytes(8, byteorder='little')
        # if self.hash not is binary, convert it to binary
        else:
            self.hash = self.hash.to_bytes(8, byteorder='little')
        super(Translated_Content, self).save(*args, **kwargs)




import httpx
from .base import TranslatorEngine
import logging
from django.db import models
from encrypted_model_fields.fields import EncryptedCharField
from django.utils.translation import gettext_lazy as _

class MicrosoftTranslator(TranslatorEngine):
    # https://learn.microsoft.com/en-us/azure/ai-services/translator/language-support
    api_key = EncryptedCharField(_("API Key"), max_length=255)
    location = models.CharField(max_length=100)
    endpoint = models.CharField(max_length=255, default="https://api.cognitive.microsofttranslator.com")
    max_characters = models.IntegerField(default=5000)
    language_code_map = {
        "English": "en",
        "Chinese Simplified": "zh-Hans",
        "Chinese Traditional": "zh-Hant",
        "Russian": "ru",
        "Japanese": "ja",
        "Korean": "ko",
        "Czech": "cs",
        "Danish": "da",
        "German": "de",
        "Spanish": "es",
        "French": "fr",
        "Indonesian": "id",
        "Italian": "it",
        "Hungarian": "hu",
        "Norwegian Bokmål": "nb",
        "Dutch": "nl",
        "Polish": "pl",
        "Portuguese": "pt-pt",
        "Swedish": "sv",
        "Turkish": "tr",
    }

    class Meta:
        verbose_name = "Microsoft Translator"
        verbose_name_plural = "Microsoft Translator"

    def validate(self) -> bool:
        result = self.translate("Hi", "Chinese Simplified")
        return result.get("result") != ""

    def translate(self, text:str, target_language:str) -> dict:
        logging.info(">>> Microsoft Translate [%s]: %s", target_language, text)
        target_code = self.language_code_map.get(target_language, None)
        translated_text = ''
        try:
            if target_code is None:
                logging.error("MicrosoftTranslator->Not support target language:%s", target_language)

            constructed_url = f"{self.endpoint}/translate"
            params = {"api-version": "3.0", "to": target_code}
            headers = {
                "Ocp-Apim-Subscription-Key": self.api_key,
                "Ocp-Apim-Subscription-Region": self.location,
                "Content-type": "application/json",
                "X-ClientTraceId": str(uuid.uuid4()),
            }
            body = [{"text": text}]

            with httpx.Client() as client:
                resp = client.post(
                    constructed_url, params=params, headers=headers, json=body, timeout=10
                )
                resp.raise_for_status()
                translated_text = resp.json()[0]["translations"][0]["text"]
            # [{'detectedLanguage': {'language': 'en', 'score': 1.0}, 'translations': [{'text': '你好，我叫约翰。', 'to': 'zh-Hans'}]}]
        except Exception as e:
            logging.error("MicrosoftTranslator->%s: %s", e, text)
        finally:
            return {'text': translated_text, "characters": len(text)}




from PyDeepLX import PyDeepLX
from .base import TranslatorEngine
import logging
from time import sleep
from django.db import models
from django.utils.translation import gettext_lazy as _

class DeepLWebTranslator(TranslatorEngine):
    # https://github.com/OwO-Network/PyDeepLX
    max_characters = models.IntegerField(default=5000)
    interval = models.IntegerField(_("Request Interval(s)"), default=5)
    proxy = models.URLField(_("Proxy(optional)"), null=True, blank=True, default=None)
    language_code_map = {
        "English": "EN",
        "Chinese Simplified": "ZH",
        "Russian": "RU",
        "Japanese": "JA",
        "Korean": "KO",
        "Czech": "CS",
        "Danish": "DA",
        "German": "DE",
        "Spanish": "ES",
        "French": "FR",
        "Indonesian": "ID",
        "Italian": "IT",
        "Hungarian": "HU",
        "Norwegian Bokmål": "NB",
        "Dutch": "NL",
        "Polish": "PL",
        "Portuguese": "PT-PT",
        "Swedish": "SV",
        "Turkish": "TR",
    }

    class Meta:
        verbose_name = "DeepL Web"
        verbose_name_plural = "DeepL Web"

    def validate(self) -> bool:
        try:
            resp = self.translate("Hello World", "Chinese Simplified")
            return resp.get("result") != ""
        except Exception as e:
            return False

    def translate(self, text:str, target_language:str) -> dict:
        logging.info(">>> DeepL Web Translate [%s]: %s", target_language, text)
        target_code = self.language_code_map.get(target_language, None)
        translated_text = ''
        try:
            if target_code is None:
                logging.error("DeepLWebTranslator->Not support target language:%s", target_language)

            translated_text = PyDeepLX.translate(text=text, targetLang=target_code, sourceLang="auto",
                                                 proxies=self.proxy)
        except Exception as e:
            logging.error("DeepLWebTranslator->%s: %s", e, text)
        finally:
            sleep(self.interval)
            return {'text': translated_text, "characters": len(text)}



# Generated by Django 5.0.1 on 2024-01-25 12:00

import encrypted_model_fields.fields
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ('translator', '0002_openaitranslator_base_url_and_more'),
    ]

    operations = [
        migrations.CreateModel(
            name='AzureAITranslator',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=100, unique=True, verbose_name='Name')),
                ('valid', models.BooleanField(null=True, verbose_name='Valid')),
                ('api_key', encrypted_model_fields.fields.EncryptedCharField(verbose_name='API Key')),
                ('endpoint', models.URLField(default='https://example.openai.azure.com/', verbose_name='Endpoint')),
                ('version', models.CharField(default='2023-12-01-preview', max_length=50)),
                ('deloyment_name', models.CharField(max_length=100)),
                ('prompt', models.TextField(
                    default='Translate the following to {target_language},only returns translations.\n{text}')),
                ('temperature', models.FloatField(default=0.5)),
                ('top_p', models.FloatField(default=0.95)),
                ('frequency_penalty', models.FloatField(default=0)),
                ('presence_penalty', models.FloatField(default=0)),
                ('max_tokens', models.IntegerField(default=1000)),
            ],
            options={
                'verbose_name': 'Azure OpenAI',
                'verbose_name_plural': 'Azure OpenAI',
            },
        ),
        migrations.AddField(
            model_name='openaitranslator',
            name='frequency_penalty',
            field=models.FloatField(default=0),
        ),
        migrations.AddField(
            model_name='openaitranslator',
            name='presence_penalty',
            field=models.FloatField(default=0),
        ),
        migrations.AddField(
            model_name='openaitranslator',
            name='top_p',
            field=models.FloatField(default=0.95),
        ),
    ]



# Generated by Django 5.0.2 on 2024-02-18 07:04

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ('translator', '0012_geminitranslator_interval'),
    ]

    operations = [
        migrations.CreateModel(
            name='DeepLWebTranslator',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=100, unique=True, verbose_name='Name')),
                ('valid', models.BooleanField(null=True, verbose_name='Valid')),
                ('max_characters', models.IntegerField(default=50000)),
                ('interval', models.IntegerField(default=5, verbose_name='Request Interval(s)')),
                ('proxy', models.URLField(blank=True, default=None, null=True, verbose_name='Proxy(optional)')),
            ],
            options={
                'verbose_name': 'DeepL Web',
                'verbose_name_plural': 'DeepL Web',
            },
        ),
    ]



# Generated by Django 5.0.2 on 2024-02-17 00:43

import encrypted_model_fields.fields
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ('translator', '0009_deepltranslator_proxy_deepltranslator_server_url'),
    ]

    operations = [
        migrations.CreateModel(
            name='ClaudeTranslator',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=100, unique=True, verbose_name='Name')),
                ('valid', models.BooleanField(null=True, verbose_name='Valid')),
                ('model', models.CharField(
                    choices=[('claude-instant-1.2', 'claude-instant-1.2'), ('claude-2.1', 'claude-2.1'),
                             ('claude-2.0', 'claude-2.0')], default='claude-instant-1.2', max_length=50)),
                ('api_key', encrypted_model_fields.fields.EncryptedCharField(verbose_name='API Key')),
                ('max_tokens', models.IntegerField(default=1000)),
                ('base_url', models.URLField(default='https://api.anthropic.com', verbose_name='API URL')),
                ('prompt', models.TextField(
                    default='Translate only the text from the following into {target_language},only returns translations.\n{text}')),
                ('proxy', models.URLField(blank=True, default=None, null=True, verbose_name='Proxy(optional)')),
                ('temperature', models.FloatField(default=0.7)),
                ('top_p', models.FloatField(blank=True, default=0.7, null=True)),
                ('top_k', models.IntegerField(default=1)),
            ],
            options={
                'verbose_name': 'Anthropic Claude',
                'verbose_name_plural': 'Anthropic Claude',
            },
        ),
    ]



from openai import OpenAI
from .base import TranslatorEngine
import logging
from django.db import models
from encrypted_model_fields.fields import EncryptedCharField
from django.utils.translation import gettext_lazy as _

class MoonshotAITranslator(TranslatorEngine):
    # https://platform.moonshot.cn/docs/api-reference
    moonshotai_models = [
        "moonshot-v1-8k",
        "moonshot-v1-32k",
        "moonshot-v1-128k",
    ]
    api_key = EncryptedCharField(_("API Key"), max_length=255)
    base_url = models.URLField(_("API URL"), default="https://api.moonshot.cn/v1")
    model = models.CharField(max_length=100, default="moonshot-v1-8k", choices=[(x, x) for x in moonshotai_models])
    prompt = models.TextField(
        default="Translate only the text from the following into {target_language},only returns translations.\n{text}")
    temperature = models.FloatField(default=0.5)
    top_p = models.FloatField(default=0.95)
    frequency_penalty = models.FloatField(default=0)
    presence_penalty = models.FloatField(default=0)
    max_tokens = models.IntegerField(default=1000)

    class Meta:
        verbose_name = "Moonshot AI"
        verbose_name_plural = "Moonshot AI"

    def _init(self):
        return OpenAI(
                    api_key=self.api_key,
                    base_url = self.base_url,
                    timeout=180.0,
                )

    def validate(self) -> bool:
        if self.api_key:
            try:
                client = self._init()
                res = client.with_options(max_retries=3).chat.completions.create(
                    model=self.model,
                    messages=[{"role": "user", "content": 'Hi'}],
                    max_tokens=10,
                )
                return True
            except Exception as e:
                return False

    def translate(self, text:str, target_language:str) -> dict:
        logging.info(">>> Moonshot AI Translate [%s]:", target_language)
        client = self._init()
        tokens = 0
        translated_text = ''
        try:
            prompt = self.prompt.format(target_language=target_language, text=text)
            res = client.with_options(max_retries=3).chat.completions.create(
                model=self.model,
                messages=[{"role": "user", "content": prompt}],
                temperature=self.temperature,
                top_p=self.top_p,
                frequency_penalty=self.frequency_penalty,
                presence_penalty=self.presence_penalty,
                max_tokens=self.max_tokens,
            )
            if res.choices[0].finish_reason == "stop":
                translated_text = res.choices[0].message.content
            else:
                translated_text = ''
                logging.info("OpenAITranslator->%s: %s", res.choices[0].finish_reason, text)
            tokens = res.usage.total_tokens
        except Exception as e:
            logging.error("OpenAITranslator->%s: %s", e, text)

        return {'text': translated_text, "tokens": tokens}


from .base import TranslatorEngine, Translated_Content
from .azureai import AzureAITranslator
from .caiyun import CaiYunTranslator
from .claude import ClaudeTranslator
from .deepl import DeepLTranslator
from .deeplx import DeepLXTranslator
from .deeplweb import DeepLWebTranslator
from .gemini import GeminiTranslator
from .google_translate_web import GoogleTranslateWebTranslator
from .microsoft_translate import MicrosoftTranslator
from .moonshotai import MoonshotAITranslator
from .openai import OpenAITranslator
from .dev import TestTranslator



# Generated by Django 5.0.2 on 2024-02-29 06:25

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("translator", "0016_alter_deeplwebtranslator_max_characters_and_more"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="translated_content",
            name="id",
        ),
        migrations.AlterField(
            model_name="translated_content",
            name="hash",
            field=models.BinaryField(
                max_length=8, primary_key=True, serialize=False, unique=True
            ),
        ),
    ]



import anthropic
from .base import TranslatorEngine
import logging
from django.db import models
from encrypted_model_fields.fields import EncryptedCharField
from django.utils.translation import gettext_lazy as _

class ClaudeTranslator(TranslatorEngine):
    # https://docs.anthropic.com/claude/reference/getting-started-with-the-api
    claude_models = ['claude-instant-1.2', 'claude-2.1', 'claude-2.0', 'claude-3-opus-20240229', 'claude-3-sonnet-20240229']
    model = models.CharField(max_length=50, default="claude-instant-1.2", choices=[(x, x) for x in claude_models])
    api_key = EncryptedCharField(_("API Key"), max_length=255)
    max_tokens = models.IntegerField(default=1000)
    base_url = models.URLField(_("API URL"), default="https://api.anthropic.com")
    prompt = models.TextField(
        default="Translate only the text from the following into {target_language},only returns translations.\n{text}")
    proxy = models.URLField(_("Proxy(optional)"), null=True, blank=True, default=None)
    temperature = models.FloatField(default=0.7)
    top_p = models.FloatField(null=True, blank=True, default=0.7)
    top_k = models.IntegerField(default=1)

    class Meta:
        verbose_name = "Anthropic Claude"
        verbose_name_plural = "Anthropic Claude"

    def _init(self):
        return anthropic.Anthropic(
            api_key=self.api_key,
            base_url=self.base_url,
            proxies=self.proxy,
        )

    def validate(self) -> bool:
        if self.api_key:
            try:
                res = self.translate("hi", "Chinese Simplified")
                return res.get("result") != ""
            except Exception as e:
                return False

    def translate(self, text:str, target_language:str) -> dict:
        logging.info(">>> Claude Translate [%s]:", target_language)
        client = self._init()
        tokens = client.count_tokens(text)
        translated_text = ''
        try:
            prompt = self.prompt.format(target_language=target_language, text=text)
            res = client.messages.create(
                model=self.model,
                max_tokens=self.max_tokens,
                messages=[{"role": "user", "content": prompt}],
                temperature=self.temperature,
                top_p=self.top_p,
                top_k=self.top_k,
            )
            result = res.content
            if result[0].type == "text":
                translated_text = result[0].text
                tokens += res.usage.output_tokens
        except Exception as e:
            logging.error("ClaudeTranslator->%s: %s", e, text)
        finally:
            return {'text': translated_text, "tokens": tokens}



# Generated by Django 5.0.2 on 2024-02-17 12:22

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ('translator', '0010_claudetranslator'),
    ]

    operations = [
        migrations.CreateModel(
            name='GoogleTranslateWebTranslator',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=100, unique=True, verbose_name='Name')),
                ('valid', models.BooleanField(null=True, verbose_name='Valid')),
                ('base_url',
                 models.URLField(default='https://translate.googleapis.com/translate_a/t', verbose_name='URL')),
                ('proxy', models.URLField(blank=True, default=None, null=True, verbose_name='Proxy(optional)')),
                ('interval', models.IntegerField(default=3, verbose_name='Request Interval(s)')),
            ],
            options={
                'verbose_name': 'Google Translate(Web)',
                'verbose_name_plural': 'Google Translate(Web)',
            },
        ),
        migrations.AddField(
            model_name='deeplxtranslator',
            name='interval',
            field=models.IntegerField(default=3, verbose_name='Request Interval(s)'),
        ),
    ]
